'''
Data translators

Some `DataSource` and `DataTranslator` types. Some deal with generic file types (e.g., comma-separated values) while
others are specific to the format of a kind of file housed in owmeta.

'''
